package main

import (
	"fmt"
	"github.com/tencentcloud/tencentcloud-sdk-go/tencentcloud/common"
	"github.com/tencentcloud/tencentcloud-sdk-go/tencentcloud/common/errors"
	"github.com/tencentcloud/tencentcloud-sdk-go/tencentcloud/common/profile"
	tdmq "github.com/tencentcloud/tencentcloud-sdk-go/tencentcloud/tdmq/v20200217"
)

/**
 * 完整示例demo
 * 创建topic -> 创建订阅 -> 发送消息 -> 消费消息 -> 确认消息
 */
func main() {
	/**
	 * 腾讯云账户SecretId
	 * 密钥可前往https://console.cloud.tencent.com/cam/capi网站进行获取
	 */
	const SECRET_ID = "AKIDSiiRt87oxxxxxxxo55oH"

	/**
	 * 腾讯云账户SecretKey
	 */
	const SECRET_KEY = "GGzSeaM5xxxxxxxxx6LIzPYQ2O"

	/**
	 * 云api地址
	 */
	const ENDPOINT = "tdmq.tencentcloudapi.com"

	/**
	 * 授权角色名称
	 */
	const USERNAME = "admin"

	/**
	 * 授权角色密钥
	 */
	const TOKEN = "eyJrZXlJZC......"

	/**
	 * 集群所属地域参数：参考：https://cloud.tencent.com/document/api/1179/46067#.E5.9C.B0.E5.9F.9F.E5.88.97.E8.A1.A8
	 */
	const REGION = "ap-shanghai"

	// 集群id
	var clusterId = "pulsar-xxx"
	// 命名空间名称
	var environmentId = "go_http"
	// topic名称
	var topicName = "topic1"
	// 订阅名称
	var sub = "sub"

	// topic全称
	var fullNameTopic = clusterId + "/" + environmentId + "/" + topicName

	// 认证信息
	credential := common.NewCredential(SECRET_ID, SECRET_KEY)
	cpf := profile.NewClientProfile()
	cpf.HttpProfile.Endpoint = ENDPOINT
	// 创建tdmq客户端
	client, _ := tdmq.NewClient(credential, REGION, cpf)

	// 创建topic
	topicRequest := tdmq.NewCreateTopicRequest()
	topicRequest.EnvironmentId = common.StringPtr(environmentId)
	topicRequest.TopicName = common.StringPtr(topicName)
	topicRequest.Partitions = common.Uint64Ptr(3)
	topicRequest.Remark = common.StringPtr("topic1说明信息")
	// 设置topic类型0： 普通消息； 1 ：全局顺序消息(设置全局顺序是，分区数要为0)； 2 ：局部顺序消息；
	topicRequest.TopicType = common.Uint64Ptr(0)
	topicRequest.ClusterId = common.StringPtr(clusterId)
	topicResponse, err := client.CreateTopic(topicRequest)
	if _, ok := err.(*errors.TencentCloudSDKError); ok {
		fmt.Printf("An API error has returned: %s\n", err)
		return
	}
	if err != nil {
		panic(err)
	}
	fmt.Printf("%s\n", topicResponse.ToJsonString())

	// 创建topic
	subRequest := tdmq.NewCreateSubscriptionRequest()
	subRequest.EnvironmentId = common.StringPtr(environmentId)
	subRequest.TopicName = common.StringPtr(topicName)
	subRequest.SubscriptionName = common.StringPtr(sub)
	subRequest.Remark = common.StringPtr("订阅说明")
	subRequest.IsIdempotent = common.BoolPtr(true)
	subRequest.ClusterId = common.StringPtr(clusterId)
	// 是否自动创建死信和重试主题，True 表示创建，False表示不创建，默认自动创建死信和重试主题。
	subRequest.AutoCreatePolicyTopic = common.BoolPtr(false)
	// 指定死信和重试主题名称规范，LEGACY表示历史命名规则，COMMUNITY表示Pulsar社区命名规范
	subRequest.PostFixPattern = common.StringPtr("COMMUNITY")
	subResponse, err := client.CreateSubscription(subRequest)
	if _, ok := err.(*errors.TencentCloudSDKError); ok {
		fmt.Printf("An API error has returned: %s\n", err)
		return
	}
	if err != nil {
		panic(err)
	}
	fmt.Printf("%s\n", subResponse.ToJsonString())

	// 发送消息
	sendRequest := tdmq.NewSendMessagesRequest()

	sendRequest.StringToken = common.StringPtr(TOKEN)
	sendRequest.ProducerName = common.StringPtr(USERNAME)
	sendRequest.Topic = common.StringPtr(fullNameTopic)
	sendRequest.Payload = common.StringPtr("this is a new message.")
	sendRequest.SendTimeout = common.Int64Ptr(5000)

	sendResponse, err := client.SendMessages(sendRequest)
	if _, ok := err.(*errors.TencentCloudSDKError); ok {
		fmt.Printf("An API error has returned: %s\n", err)
		return
	}
	if err != nil {
		panic(err)
	}
	fmt.Printf("%s\n", sendResponse.ToJsonString())

	// 消费消息
	receiveRequest := tdmq.NewReceiveMessageRequest()

	receiveRequest.Topic = common.StringPtr(fullNameTopic)
	receiveRequest.SubscriptionName = common.StringPtr(sub)
	receiveRequest.ReceiverQueueSize = common.Int64Ptr(10)
	receiveRequest.SubInitialPosition = common.StringPtr("Latest")

	receiveResponse, err := client.ReceiveMessage(receiveRequest)
	if _, ok := err.(*errors.TencentCloudSDKError); ok {
		fmt.Printf("An API error has returned: %s\n", err)
		return
	}
	if err != nil {
		panic(err)
	}
	fmt.Printf("%s\n", receiveResponse.ToJsonString())

	// 确认消息
	acknowledgeRequest := tdmq.NewAcknowledgeMessageRequest()

	acknowledgeRequest.MessageId = receiveResponse.Response.MessageID
	acknowledgeRequest.AckTopic = common.StringPtr(fullNameTopic)
	acknowledgeRequest.SubName = common.StringPtr(sub)

	acknowledgeResponse, err := client.AcknowledgeMessage(acknowledgeRequest)
	if _, ok := err.(*errors.TencentCloudSDKError); ok {
		fmt.Printf("An API error has returned: %s\n", err)
		return
	}
	if err != nil {
		panic(err)
	}
	fmt.Printf("%s\n", acknowledgeResponse.ToJsonString())
}
