package main

import (
	"fmt"

	"github.com/tencentcloud/tencentcloud-sdk-go/tencentcloud/common"
	"github.com/tencentcloud/tencentcloud-sdk-go/tencentcloud/common/errors"
	"github.com/tencentcloud/tencentcloud-sdk-go/tencentcloud/common/profile"
	tdmq "github.com/tencentcloud/tencentcloud-sdk-go/tencentcloud/tdmq/v20200217"
)

/**
 * 批量发送消息示例
 */
func main() {

	// 认证信息
	credential := common.NewCredential(
		"AKIDSiiRt87oxxxxxxxo55oH",
		"GGzSeaM5xxxxxxxxx6LIzPYQ2O",
	)
	cpf := profile.NewClientProfile()
	cpf.HttpProfile.Endpoint = "tdmq.tencentcloudapi.com"
	// 创建客户端
	client, _ := tdmq.NewClient(credential, "ap-shanghai", cpf)

	request := tdmq.NewSendBatchMessagesRequest()
	// 已授权角色密钥
	request.StringToken = common.StringPtr("eyJrZXlJZC......")
	// 已授权角色名称
	request.ProducerName = common.StringPtr("admin")
	// 设置topic名称, 格式为:集群（租户）ID/命名空间/Topic名称
	request.Topic = common.StringPtr("pulsar-xxx/go_http/topic1")
	// 消息内容
	request.Payload = common.StringPtr("this is a new message.")
	// 发送超时时间
	request.SendTimeout = common.Int64Ptr(3000)
	// 设置内存中允许缓存的生产消息的最大数量
	request.MaxPendingMessages = common.Int64Ptr(10)
	// 设置每一个batch中消息的最大数量
	request.BatchingMaxMessages = common.Int64Ptr(10)
	// 设置每一个batch最大等待的时间，超过这个时间，不管是否达到指定的batch中消息的数量和大小，都会将该batch发送出去，
	request.BatchingMaxPublishDelay = common.Int64Ptr(1000)
	// 设置每一个batch中最大允许的消息的大小
	request.BatchingMaxBytes = common.Int64Ptr(1024)
	// 发送消息
	response, err := client.SendBatchMessages(request)
	if _, ok := err.(*errors.TencentCloudSDKError); ok {
		fmt.Printf("An API error has returned: %s", err)
		return
	}
	if err != nil {
		panic(err)
	}
	fmt.Printf("%s", response.ToJsonString())
}
