package main

import (
	"fmt"

	"github.com/tencentcloud/tencentcloud-sdk-go/tencentcloud/common"
	"github.com/tencentcloud/tencentcloud-sdk-go/tencentcloud/common/errors"
	"github.com/tencentcloud/tencentcloud-sdk-go/tencentcloud/common/profile"
	tdmq "github.com/tencentcloud/tencentcloud-sdk-go/tencentcloud/tdmq/v20200217"
)

/**
 * 创建订阅示例
 */
func main() {
	// 认证信息
	credential := common.NewCredential(
		"AKIDSiiRt87oxxxxxxxo55oH",
		"GGzSeaM5xxxxxxxxx6LIzPYQ2O",
	)
	cpf := profile.NewClientProfile()
	cpf.HttpProfile.Endpoint = "tdmq.tencentcloudapi.com"
	// 创建tdmq客户端
	client, _ := tdmq.NewClient(credential, "ap-shanghai", cpf)

	request := tdmq.NewCreateSubscriptionRequest()

	// 设置命名空间名称
	request.EnvironmentId = common.StringPtr("go_http")
	// 设置topic名称
	request.TopicName = common.StringPtr("topic1")
	// 设置订阅名称
	request.SubscriptionName = common.StringPtr("sub")
	// 设置订阅说明
	request.Remark = common.StringPtr("订阅说明")
	// 是否幂等创建，若否不允许创建同名的订阅关系。
	request.IsIdempotent = common.BoolPtr(true)
	// 设置集群id
	request.ClusterId = common.StringPtr("pulsar-xxx")
	// 是否自动创建死信和重试主题，True 表示创建，False表示不创建，默认自动创建死信和重试主题。
	request.AutoCreatePolicyTopic = common.BoolPtr(false)
	// 指定死信和重试主题名称规范，LEGACY表示历史命名规则，COMMUNITY表示Pulsar社区命名规范
	request.PostFixPattern = common.StringPtr("COMMUNITY")

	// 创建订阅信息
	response, err := client.CreateSubscription(request)
	if _, ok := err.(*errors.TencentCloudSDKError); ok {
		fmt.Printf("An API error has returned: %s", err)
		return
	}
	if err != nil {
		panic(err)
	}
	fmt.Printf("%s", response.ToJsonString())
}
